import fs from "node:fs";
import path from "node:path";
import yaml from "js-yaml";
const loadYaml = (f) => yaml.load(fs.readFileSync(path.resolve(f), "utf8"));

export function getFtpContainer(serverSpec) {
  const { mclName, storage } = serverSpec;
  const ftpContainer = loadYaml("lib/k8s/configs/containers/ftp-server.yml");
  ftpContainer.name = `mcl-${mclName}-ftp`;
  const ftpPortList = [
    { p: 20, n: "ftp-data" },
    { p: 21, n: "ftp-commands" },
  ];
  for (var p = 40000; p <= 40009; p++)
    ftpPortList.push({ p, n: `ftp-passive-${p - 40000}` });
  ftpContainer.ports = ftpPortList.map(({ p: containerPort, n: name }) => ({
    containerPort,
    name,
    protocol: "TCP",
  }));
  if (!storage) delete ftpContainer.volumeMounts;
  return ftpContainer;
}

export function getCoreServerContainer(serverSpec) {
  const { mclName, version, serverType, memory, storage } = serverSpec;
  const container = loadYaml("lib/k8s/configs/containers/minecraft-server.yml");
  // Container Updates
  container.name = `mcl-${mclName}-server`;
  container.resources.requests.memory = `${memory}Mi`;

  const findEnv = (k) => container.env.find(({ name: n }) => n === k);
  const updateEnv = (k, v) => (findEnv(k).value = v);

  // Enviornment variables
  updateEnv("TYPE", serverType);
  updateEnv("VERSION", version);
  updateEnv("MEMORY", `${memory}M`);
  // RCON
  const rs = `mcl-${mclName}-rcon-secret`;
  findEnv("RCON_PASSWORD").valueFrom.secretKeyRef.name = rs;
  if (!storage) delete container.volumeMounts;
  return container;
}

export function getServerContainer(serverSpec) {
  const {
    difficulty,
    gamemode,
    motd,
    maxPlayers,
    seed,
    ops,
    whitelist,
    storage,
  } = serverSpec;
  const container = getCoreServerContainer(serverSpec);

  const findEnv = (k) => container.env.find(({ name: n }) => n === k);
  const updateEnv = (k, v) => (findEnv(k).value = v);

  // Enviornment variables
  /*updateEnv("DIFFICULTY", difficulty);
  updateEnv("MODE", gamemode);
  updateEnv("MOTD", motd);
  updateEnv("MAX_PLAYERS", maxPlayers);
  updateEnv("SEED", seed);
  updateEnv("OPS", ops);
  updateEnv("WHITELIST", whitelist); */
  if (!storage) delete container.volumeMounts;

  return container;
}

export function getBackupContainer(serverSpec) {
  const { mclName, backupEnabled, backupPath, storage } = serverSpec;
  const container = loadYaml("lib/k8s/configs/containers/minecraft-backup.yml");
  if (!backupEnabled) return;
  const findEnv = (k) => container.env.find(({ name: n }) => n === k);
  const updateEnv = (k, v) => (findEnv(k).value = v);
  updateEnv("RCLONE_REMOTE", `${mclName}-backup`);
  updateEnv("RCLONE_DEST_DIR", backupPath);
  container.name = `mcl-${mclName}-backup`;
  // RCON
  const rs = `mcl-${mclName}-rcon-secret`;
  findEnv("RCON_PASSWORD").valueFrom.secretKeyRef.name = rs;
  if (!storage) delete container.volumeMounts;

  return container;
}
